//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function r = SerialLink(L,varargin)

    // Numero di argomenti nella chiamata di funzione
    [%nargout,%nargin] = argn(0);

    r = tlist(["Robot","name","manuf","comment","links","n","mdh","gravity","base","tool","handles","lineopt","shadowopt","plotopt","offset","qlim","q","config"]);
    //missing proprieties:
    //        
    //        qteach
    //

    r.name = "noname";
    r.manuf = "";
    r.comment = "";
    r.links = [];
    r.n = 0; //r.n = 0;
    r.mdh = 0;
    r.gravity = [0;0;9.81];
    r.base = eye(4,4);
    r.tool = eye(4,4);
    r.handles = [];
    r.config = [];
    // TO DO: lineopt, shadowopt, plotopt accessible through specific methods or r(11)r(12),r(13)
    r.lineopt = list('black', 7); // r.lineopt = list(color, thickness) //list('black', 7);
    r.shadowopt = list('grey70', 3);//'Color''Linewidth'
    r.plotopt =  list();

    r.offset = [];
    r.qlim = [];
    r.q=[];

    if %nargin==0 then
        // zero argument constructor, sets default values
        return;
    else
        // at least one argument, either a robot or link array
        if typeof(L) == "Robot"
				// clone the passed robot
				r = SerialLink();
				r=L;
        // SerialLink has received a list of "Link"" or "Robot"" objects 
        elseif typeof(L(1))=='Link'
            r.links = L;    // attach the links
        elseif  typeof(L(1)) =='Robot'
            if length(L) == 1 then
				// clone the passed robot
				r = SerialLink();
				r=L;
			else
				r = L(1);
				for k=2:length(L)
					r = r * L(k);
				end
			end
            //        // legacy matrix
            //        dh_dyn = L;
            //        clear L
            //        for j=1:numrows(dh_dyn)
            //            L(j) =        Link();
            //            L(j).theta =  dh_dyn(j,1);
            //            L(j).d =      dh_dyn(j,2);
            //            L(j).a =      dh_dyn(j,3);
            //            L(j).alpha =  dh_dyn(j,4);
            //
            //            if numcols(dh_dyn) > 4
            //                L(j).sigma = dh_dyn(j,5);
            //            end
            //        end
            //        r.links = L;
        else
            error('SERIALLINK: unknown type passed to robot');
        end
        r.n = length(r.links);
    end

    //process the rest of the arguments in key, value pairs
    // varargin(i)      i odd -> field
    //                  i even -> string
    for i=1:length(varargin)
        if  modulo(i,2) <> 0 then
            field = varargin(i);
            select field
            case "name" 
                r.name = varargin(i+1);
            case "manuf" 
                r.manuf = varargin(i+1);
            case "manufacturer" 
                r.manuf = varargin(i+1);
            case "comment"
                r.comment = varargin(i+1);
            case "gravity"
                if length(varargin(i+1)) == 3 then
                    r.gravity = varargin(i+1);
                else
                    error("SERIALLINK: gravity must be a 3-elements vector");
                end
            case "base"
                if ~ishomog(varargin(i+1)) then
                    error("SERIALLINK: base must be a homogeneous transform");
                else
                    r.base=varargin(i+1);
                end
            case "tool"
                if ~ishomog(varargin(i+1)) then
                    error("SERIALLINK: tool must be a homogeneous transform");
                else
                    r.tool=varargin(i+1);
                end
            case "offset"
                v = varargin(i+1)
                if and(size(v) == [1, r.n]) then
                    L = r.links;
                    for j=1:length(L)
                        L(j).offset = v(i);
                    end
                    r.links = L;
				else
                    msg = gettext("SERIALLINK: joint offset vector must be 1x%d.\n")
                    error(msprintf(msg, r.n))
				end
            case "qlim"
                v = varargin(i+1);
                if size(v) ~= [r.n, 2] then
                    msg = gettext("SERIALLINK: joint limit matrix must be %dx2.\n")
                    error(msprintf(msg, r.n))
				end
				L = r.links;
				for i=1:r.n
					L(i).qlim = v(i,:);
				end
				r.links = L; 
			case "q"
                v = varargin(i+1);
				if and(size(v) == [1, r.n]) then
					r.q=v;
				else
                    msg = gettext("SERIALLINK: joint vector must be 1x%d.\n")
                    error(msprintf(msg, r.n))
				end
            case "plotopt"
                r.plotopt = varargin(i+1);
            case "lineopt"
                r.lineopt = varargin(i+1);
            case "shadowopt"
                r.shadowopt = varargin(i+1);
            else 
                mprintf("\nUnknown option: %s\nOptions:  name,  manuf,  comment,  base,  tool, gravity,  offset,  qlim,  plotopt, lineopt, shadowopt \n\n", varargin(i));
            end
        end
    end
        // set r.config
    if typeof(L(1)) == "Link" then
        r.config = RP(L)
    end
    
    // set the robot object mdh status flag
    for i=1:r.n,
        mdh(i) = [r.links(i).mdh];
    end
    if and(mdh==0) then
        r.mdh = mdh(1);
    elseif and(mdh == 1) then
        r.mdh = mdh(1);
    else
        error("SERIALLINK: robot has mixed D&H links conventions");
    end;

endfunction
