//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function T = eul2tr(alpha, varargin)
    // Euler angles to homogeneous transform
    //
    // Calling Sequence
    // T = eul2tr(ANG, OPTIONS)
    // T = eul2tr(ALPHA, BETA, GAMMA, OPTIONS)
    //
    // Description
    // T = EUL2TR(ANG, OPTIONS) is a homogeneous transformation equivalent to the 
    // specified alpha, beta, gamma angles which correspond to rotations about the 
    // X, Y, Z mobile axes respectively (Euler angles). If ANG has multiple rows they are assumed to 
    // represent a trajectory and T is a three dimensional matrix, where the last index  
    // corresponds to the rows of ANG.
    //
    // <refsection>
    // <title>Options</title>
    // <variablelist>
    //   <varlistentry><term>'deg'     :</term>
    // 	<listitem><para>Compute angles in degrees (radians default)</para></listitem></varlistentry>
    //   <varlistentry><term>'c'       :</term>
    // 	<listitem><para>Computes TR by means of a compiled c-function (faster)</para></listitem></varlistentry>
    // </variablelist>
    // </refsection>
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem>
    //   <para>
    //     Input angles are handled as Euler angles
    //     and the rotation matrix is evaluated as follows
    //   </para>
    //   <para>
    //   </para>
    //   <para>
    //     R = Rx(alpha)*Ry(beta)*Rz(gamma).
    //   </para>
    // </listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also
    // tr2eul
    // rpy2tr
    // tr2rpy
    // rts_c_rpy2tr
    // rts_c_eul2tr
    // rpy2r 
    // zyz2tr
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci
    // Copyright (C) 2020 - Corrado Guarino Lo Bianco

    [%nargout,%nargin] = argn(0);

    flag_c = %f
    out =list();
    for i = 1:(%nargin-1),
        if ~strcmp(string(varargin(i)),'c') then
            flag_c = %t;
        else
            out($+1) = varargin(i);
        end
    end
    
    if flag_c then
        try
            T = rts_c_eul2tr(alpha,out(:));
            T = matrix(T,[4 4 numrows(alpha)]);
            return;
        catch
            warning(msprintf(gettext("%s: rts_c_eul2tr not found.\n"),"eul2tr"));
            R = eul2r(alpha, out(:));
            T = r2t(R);
            return;
        end
    end

    R = eul2r(alpha, out(:));
    T = r2t(R);

endfunction
