//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function t = fkine(robot,q)
    // Forward kinematics
    // 
    // Calling Sequence
    // T = fkine(R,Q)
    //
    // Description
    // <literal>T = fkine(R,Q)</literal> is the pose (4x4) of the robot end-effector as a homogeneous
    // transformation for the joint configuration Q (1xN).
    // 
    // If Q is a matrix (KxN) the rows are interpretted as the generalized
    // joint coordinates for a sequence of points along a trajectory.  Q(i,j) is
    // the j''th joint parameter for the i''th trajectory point.  In this case
    // T is a 3d matrix (4x4xK) where the last subscript is the index along the path.
    // 
    // <refsection>
	// <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>The robot''s base or tool transform, if present, are incorporated into the
    //   result.</para></listitem>
    // <listitem><para>Joint offsets, if defined, are added to Q before the forward kinematics are
    //   computed.</para></listitem>
    //  </itemizedlist>
	// </refsection>
    //
    // See also 
    // SerialLink
    // ikine
    // ikine6s
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci

    t=[];
    // Numero di argomenti nella chiamata di funzione
    [%nargout,%nargin] = argn(0)
    n = robot.n;
    L = robot.links;

    if length(q) == n
        t = robot.base;
        for i=1:n
            t = t * linkTransf(L(i),q(i));
        end
        t = t * robot.tool;
    else
        if numcols(q) ~= n
            error('q must have '+sci2exp(n)+' columns');
        end
        t = zeros(4,4,0);
        for qv=q',      // for each trajectory point
            tt = robot.base;
            for i=1:n
                tt = tt * linkTransf(L(i),qv(i));
            end
            t = cat(3, t, tt * robot.tool);
        end
    end

endfunction
