//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function J0 = jacob0(robot,q,varargin)
    // Jacobian in world coordinates
    //
    // Calling Sequence
    // J0 = jacob0(R,Q, OPTIONS)
    //
    // Parameters
    // OPTIONS : valid string
    // "rpy"     Compute analytical Jacobian with rotation rate in terms of 
    //           roll-pitch-yaw angles
    // "eul"     Compute analytical Jacobian with rotation rates in terms of 
    //           Euler angles
    // "trans"   Return translational submatrix of Jacobian
    // "rot"     Return rotational submatrix of Jacobian 
    //
    // Description 
    // J0 = jacob0(R,Q, OPTIONS) is the Jacobian matrix (6xN) for the robot in 
    // pose Q (1xN).  The manipulator Jacobian matrix maps joint velocity to 
    // end-effector spatial velocity V = J0*QD expressed in the world-coordinate 
    // frame.
    // 
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>The Jacobian is computed in the world frame and transformed to the 
    //   end-effector frame.</para></listitem>
    // <listitem><para>The default Jacobian returned is often referred to as the geometric 
    //   Jacobian, as opposed to the analytical Jacobian.</para></listitem>
    // </itemizedlist>
    // </refsection>
    // 
    // See also 
    // jacobn
    // delta2tr 
    // tr2delta
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci
     
    opt.rpy = %f;
    opt.eul = %f;
    opt.trans = %f;
    opt.rot = %f;

    if ~isempty(varargin) then
        for i = 1:(length(varargin)),
            if ~strcmp(string(varargin(i)),'rpy') then
                opt.rpy = ~opt.rpy;
            end
            if ~strcmp(string(varargin(i)),'eul') then
                opt.eul = ~opt.eul;
            end
            if ~strcmp(string(varargin(i)),'trans') then
                opt.trans = ~opt.trans;
            end
            if ~strcmp(string(varargin(i)),'rot') then
                opt.rot = ~opt.rot;
            end
        end 
    end

    // 
    //   dX_tn = Jn dq
    // 
    Jn = jacobn(robot,q);   // Jacobian from joint to wrist space

    // 
    //  convert to Jacobian in base coordinates
    // 
    Tn = fkine(robot,q);    // end-effector transformation
    R = t2r(Tn);
    J0 = [R,zeros(3,3);zeros(3,3),R]*Jn;

    if opt.rpy
        rpy = tr2rpy( fkine(robot, q) );
        B = rpy2jac(rpy);
        if rcond(B) < %eps
            error('Representational singularity');
        end
        J0 = sysdiag( eye(3,3), inv(B) ) * J0;
    elseif opt.eul
        eul = tr2eul( fkine(robot, q) );
        B = eul2jac(eul);
        if rcond(B) < %eps
            error('Representational singularity');
        end
        J0 = sysdiag( eye(3,3), inv(B) ) * J0;
    end

    if opt.trans
        J0 = J0(1:3,:);
    elseif opt.rot
        J0 = J0(4:6,:);
    end
endfunction
