//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function J = jacobn(robot, q,varargin)
    // Jacobian in end-effector frame
    //
    // Calling Sequence
    // JN = jacobn(R, Q, OPTIONS)
    //
    // Parameters
    // OPTIONS : valid string
    // 'trans'  : Return translational submatrix of Jacobian
    // 'rot'    : Return rotational submatrix of Jacobian 
    //
    // Description
    // JN = jacobn(R, Q, OPTIONS) is the Jacobian matrix (6xN) for the robot in 
    // pose Q. The manipulator Jacobian matrix maps joint velocity to 
    // end-effector spatial velocity V = JN*QD in the end-effector frame.
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>This Jacobian is often referred to as the geometric Jacobian.</para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // Bibliography
    //  [1] Differential Kinematic Control Equations for Simple Manipulators, Paul, Shimano, Mayer, IEEE SMC 11(6) 1981, pp. 456-460
    //
    // See also 
    // jacob0
    // delta2tr
    // tr2delta
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci

    opt.trans = %f;
    opt.rot = %f;
    if ~isempty(varargin) then
        for i = 1:(length(varargin)),
            if ~strcmp(string(varargin(i)),'trans') then
                opt.trans = ~opt.trans;
            end
            if ~strcmp(string(varargin(i)),'rot') then
                opt.rot = ~opt.rot;
            end
        end 
    end

    n = robot.n;
    L = robot.links;       //  get the links

    //    if typeof(q)=='sym'
    //        tau(6, robot.n) = sym(); //NON PENSO VADA ANCORA
    //    else
    J = zeros(6, robot.n);
    //    end
    U = robot.tool;
    for j=n:-1:1
        if robot.mdh == 0
            // standard DH convention
            U = linkTransf(L(j),(q(j)))* U;
        end
        if RP(L(j)) == 'R'
            //disp(L(j).RP)
            //  revolute axis
            d = [   -U(1,1)*U(2,4)+U(2,1)*U(1,4)
            -U(1,2)*U(2,4)+U(2,2)*U(1,4)
            -U(1,3)*U(2,4)+U(2,3)*U(1,4)];
            delta = U(3,1:3)';  // nz oz az
        else
            // prismatic axis
            d = U(3,1:3)';      // nz oz az
            delta = zeros(3,1); //  0  0  0
        end
        J(:,j) = [d; delta];

        if robot.mdh ~= 0
            // modified DH convention
            U = linkTransf(L(j),(q(j))) * U;
        end
    end

    if opt.trans
        J = J(1:3,:);
    elseif opt.rot
        J = J(4:6,:);
    end

endfunction
