//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [w,mx] = maniplty(robot, q, varargin)
// Manipulability measure
//
// Calling Sequence
// M = maniplty(R, Q, OPTIONS)
// [M,CI] = maniplty(R, Q, OPTIONS)
//
// Parameters
// R			: robot object
// Q			: matrix or vector, joint configuration.
// OPTIONS 		: 'key',value pair or 'key'
// 'T'          : manipulability for transational motion only (default)
// 'R'          : manipulability for rotational motion only
// 'all'        : manipulability for all motions
// 'dof',D      : D is a vector (1x6) with non-zero elements if the
//               corresponding DOF is to be included for manipulability
// 'yoshikawa'  : use Yoshikawa algorithm (default)
// 'asada'      : use Asada algorithm
//
// Description
// M = maniplty(R, Q, OPTIONS) is the manipulability index measure for the robot
// at the joint configuration Q.  It indicates dexterity, that is, how isotropic 
// the robot's motion is with respect to the 6 degrees of Cartesian motion.
// The measure is high when the manipulator is capable of equal motion in all
// directions and low when the manipulator is close to a singularity.
//
// If Q is a matrix (MxN) then M (Mx1) is a vector of  manipulability 
// indices for each pose specified by a row of Q.
//
// [M,CI] = R.maniplty(R, Q, OPTIONS) as above, but for the case of the Asada
// measure returns the Cartesian inertia matrix CI.
//
// Two measures can be selected:
// <simplelist>
// <member>Yoshikawa's manipulability measure is based on the shape of the velocity
//   ellipsoid and depends only on kinematic parameters.</member>
// <member>Asada's manipulability measure is based on the shape of the acceleration
//   ellipsoid which in turn is a function of the Cartesian inertia matrix and
//   the dynamic parameters.  The scalar measure computed here is the ratio of 
//   the smallest/largest ellipsoid axis.  Ideally the ellipsoid would be 
//   spherical, giving a ratio of 1, but in practice will be less than 1.</member>
// </simplelist> 
//
// <refsection>
// <title>Notes</title>
// <itemizedlist>
// <listitem><para>By default the measure includes rotational and translational dexterity, but
// this involves adding different units.  It can be more useful to look at the
// translational and rotational manipulability separately.</para></listitem>
// </itemizedlist>
// </refsection>
//
// Bibliography
// [1] Analysis and control of robot manipulators with redundancy, T. Yoshikawa, Robotics Research: The First International Symposium (M. Brady and R. Paul, eds.), pp. 735-747, The MIT press, 1984.
// [2] A geometrical representation of manipulator dynamics and its application to arm design, H. Asada, Journal of Dynamic Systems, Measurement, and Control, vol. 105, p. 131, 1983.
//
// See also
// SerialLink
// inertia
// jacob0
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014-20 - Davide Cappucci

    [%nargout, %nargin] = argn(0);
    
    opt.method = 'yoshikawa';    //{'yoshikawa', 'asada'};
    opt.axes = 'T';      //{'T', 'all', 'R'};
    opt.dof = [];


    if ~isempty(varargin) then
        for i = 1:(length(varargin)),
            if ~strcmp(string(varargin(i)),'yoshikawa') then
                opt.method = 'yoshikawa';
            elseif ~strcmp(string(varargin(i)),'asada') then
                opt.method = 'asada';
            elseif ~strcmp(string(varargin(i)),'T') then
                opt.axes = 'T';
            elseif ~strcmp(string(varargin(i)),'all') then
                opt.axes = 'all';
            elseif ~strcmp(string(varargin(i)),'R') then
                opt.axes = 'R';
            elseif ~strcmp(string(varargin(i)),'dof')  then
                opt.dof = varargin(i+1);
            else
                warning(msprintf(gettext('%s: Unkwnon argument ''%s''\n'),'maniplty',varargin(i)));
            end
        end 
    end




    if isempty(opt.dof)
        select opt.axes
        case 'T'
            dof = [1 1 1 0 0 0];
        case 'R'
            dof = [0 0 0 1 1 1];
        case 'all'
            dof = [1 1 1 1 1 1];
        end
    else
        dof = opt.dof;
    end

    opt.dof = dof;

    if opt.method == 'yoshikawa' then
        w = zeros(numrows(q),1);
        for i=1:numrows(q)
            w(i) = yoshi(robot, q(i,:), opt);
        end
    elseif opt.method == 'asada' then
        w = zeros(numrows(q),1);
        if %nargout > 1 then
            dof = sum(opt.dof);
            MX = zeros(dof,dof,numrows(q));
            for i=1:numrows(q)
                [ww,mm] = asada(robot, q(i,:), opt);
                w(i) = ww;
                MX(:,:,i) = mm;
            end
        else
            for i=1:numrows(q)
                w(i) = asada(robot, q(i,:), opt);
            end
        end
    end

    if %nargout > 1 then
        mx = MX;
    end
endfunction
