//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [TG, taxis]  = mstraj(segments, qdmax, tsegment, q, dt, Tacc, varargin)
    // Multi-segment multi-axis trajectory
    // 
    // Calling Sequence
    // TRAJ = MSTRAJ(P, QDMAX, Q0, DT, TACC, OPTIONS)
    // TRAJ = MSTRAJ(SEGMENTS, QDMAX, Q0, DT, TACC, QD0, QDF, OPTIONS)
    //
    // Parameters
    // P :(MxN) is a matrix of via points, 1 row per via point, one column 
    //   per axis.  The last via point is the destination.
    // QDMAX :(1xN) are axis velocity limits which cannot be exceeded, or
    // QDMAX :(Mx1) are the durations for each of the M segments
    // Q0 : (1xN) are the initial axis coordinates
    // DT is the time step
    // TACC :(1x1) this acceleration time is applied to all segment transitions
    // TACC :(1xM) acceleration time for each segment, TACC(i) is the acceleration 
    //   time for the transition from segment i to segment i+1.  TACC(1) is also 
    //   the acceleration time at the start of segment 1.
    // OPTIONS : String, valid entries
    // "verbose"  :  Show details
    // "plot"   :    plot a graph.
    //
    // Description
    // <literal>TRAJ = MSTRAJ(P, QDMAX, Q0, DT, TACC, OPTIONS)</literal> is a multi-segment trajectory (KxN)
    // based on via points P (MxN) and axis velocity limits QDMAX (1xN).  The 
    // path comprises linear segments with polynomial blends.  The output 
    // trajectory matrix has one row per time step, and one column per axis.
    //
    // <literal>TRAJ = MSTRAJ(SEGMENTS, QDMAX, Q0, DT, TACC, QD0, QDF, OPTIONS)</literal> as above but 
    // additionally specifies the initial and final axis velocities (1xN).
    //
    // <refsection>
    // <title>Notes</title>
    //  <itemizedlist>
    // <listitem><para>If no output arguments are specified the trajectory is plotted.</para></listitem>
    // <listitem><para>The path length K is a function of the number of via points, Q0, DT
    //   and TACC.</para></listitem>
    // <listitem><para>The final via point P(M,:) is the destination.</para></listitem>
    // <listitem><para>The motion has M segments from Q0 to P(1,:) to P(2,:)  to P(M,:).</para></listitem>
    // <listitem><para>All axes reach their via points at the same time.</para></listitem>
    // <listitem><para>Can be used to create joint space trajectories where each axis is a joint
    //   coordinate.</para></listitem>
    // <listitem><para>Can be used to create Cartesian trajectories with the "axes" assigned
    //   to translation and orientation in RPY or Euler angle form.</para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also 
    // mtraj
    // lspb 
    // ctraj
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci

    [%nargout,%nargin] = argn(0);
    ns = numrows(segments);
    nj = numcols(segments);
    opt.verbose = %f;
    opt.plot = %f;

    if ~isempty(qdmax) & ~isempty(tsegment)
        error('Can only specify one of qdmax or tsegment');
    end
    if isempty(qdmax) & isempty(tsegment)
        error('Must specify one of qdmax or tsegment');
    end

    //    [opt,args] = tb_optparse([], varargin);
    for i=1:length(varargin)
        select varargin(i)
        case 'verbose'
            opt.verbose = %t;
        case 'plot'
            opt.plot = %T;
        end
    end

    //    if length(varargin) > 0
    //        qd0 = varargin(1);
    //    else
    qd0 = zeros(1, nj);
    //    end
    //    if length(varargin) > 1
    //        qdf = varargin(2);
    //    else
    qdf = zeros(1, nj);
    //    end

    // set the initial conditions
    q_prev = q;
    qd_prev = qd0;

    myclock = 0;      // keep track of time
    arrive = [];    // record planned time of arrival at via points

    tg = [];
    taxis = [];

    for seg=1:ns
        if opt.verbose
            mprintf('------------------- segment %d\n', seg);
        end

        // set the blend time, just half an interval for the first segment

        if length(Tacc) > 1
            tacc = Tacc(seg);
        else
            tacc = Tacc;
        end

        tacc = ceil(tacc/dt)*dt;
        tacc2 = ceil(tacc/2/dt) * dt;
        if seg == 1
            taccx = tacc2;
        else
            taccx = tacc;
        end

        // estimate travel time
        //    could better estimate distance travelled during the blend
        q_next = segments(seg,:);    // current target
        dq = q_next - q_prev;     // total distance to move this segment

        // probably should iterate over the next section to get qb right...
        //        % while 1
        //        %   qd_next = (qnextnext - qnext)
        //        %   tb = abs(qd_next - qd) ./ qddmax;
        //        %   qb = f(tb, max acceleration)
        //        %   dq = q_next - q_prev - qb
        //        %   tl = abs(dq) ./ qdmax;

        if ~isempty(qdmax)
            // qdmax is specified, compute slowest axis

            qb = taccx * qdmax / 2;        // distance moved during blend
            tb = taccx;

            // convert to time
            tl = abs(dq) ./ qdmax;
            //tl = abs(dq - qb) ./ qdmax;
            tl = ceil(tl/dt) * dt;

            // find the total time and slowest axis
            tt = tb + tl;
            [tseg,slowest] = max(tt);
            taxis(seg,:) = tt;

            // best if there is some linear motion component
            if tseg <= 2*tacc
                tseg = 2 * tacc;
            end
        elseif ~isempty(tsegment)
            // segment time specified, use that
            tseg = tsegment(seg);
            slowest = %nan;
        end

        // log the planned arrival time
        arrive(seg) = myclock + tseg;
        if seg > 1
            arrive(seg) = arrive(seg) + tacc2;
        end

        if opt.verbose
            mprintf('seg %d, slowest axis %d, time required %.4g\n', ...
            seg, slowest, tseg);
        end

        // create the trajectories for this segment

        // linear velocity from qprev to qnext
        qd = dq / tseg;

        // add the blend polynomial
        qb = jtraj(q, q_prev+tacc2*qd, 0:dt:taccx, qd_prev, qd);
        tg = [tg; qb(2:$,:)];

        myclock = myclock + taccx;     // update the myclock

        // add the linear part, from tacc/2+dt to tseg-tacc/2
        for t=tacc2+dt:dt:tseg-tacc2
            s = t/tseg;
            q = (1-s) * q_prev + s * q_next;       // linear step
            tg = [tg; q];
            myclock = myclock + dt;
        end

        q_prev = q_next;    // next target becomes previous target
        qd_prev = qd;
    end
    // add the final blend
    qb = jtraj(q, q_next, 0:dt:tacc2, qd_prev, qdf);
    tg = [tg; qb(2:$,:)];

    // plot a graph if no output argument
    if opt.plot
        t = (0:numrows(tg)-1)'*dt;
        clf
        plot(t, tg, '-o');
        drawlater()
        plot(arrive, segments, 'bo', 'MarkerFaceColor', 'k');
        drawnow();
        //        grid
        xlabel('time');
        //-----------------xaxis(t(1), t(end))
    else 
        TG = tg;
    end
endfunction
