//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function q = qinterp(Q1, Q2, r)
    // Interpolate quaternions
    //
    // Calling Sequence
    // QI = qinterp(Q1,Q2, S)
    //
    // Description
    // <literal>QI = qinterp(Q1,Q2, S)</literal> is a unit-quaternion that 
    // interpolates a rotation between Q1 for S=0 and Q2 for S=1.
    //
    // If S is a vector QI is a list of quaternions, each element
    // corresponding to sequential elements of S.
    //
    // <refsection>
	// <title>Notes</title>
	// <itemizedlist>
    // <listitem><para> This is a spherical linear interpolation (slerp) that 
    // can be interpretted as interpolation along a great circle arc on a 
    // sphere.</para></listitem>
    // <listitem><para> The value of S is clipped to the interval 0 to 1.
    // </para></listitem>
    // </itemizedlist>
	// </refsection>
    //
    // See also 
    // Quaternion
    // ctraj
    // qscale
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci

    q1 = qdouble(Q1);
    q2 = qdouble(Q2);

    theta = acos(q1*q2');
    count = 1;

    // clip values of r
    r(r<0) = 0;
    r(r>1) = 1;

    if length(r) == 1
        if theta == 0
            q = Q1;
        else
            q = Quaternion( (sin((1-r)*theta) * q1 + sin(r*theta) * q2) / sin(theta) );
        end
    else
        q = list();
        for R=r(:)'
            if theta == 0
                qq = Q1;
            else
                qq = Quaternion( (sin((1-R)*theta) * q1 + sin(R*theta) * q2) / sin(theta) );
            end
            q(count) = qq;
            count = count + 1;
        end
    end
endfunction
