//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function R = rpy2r(roll, varargin)
    // Roll-pitch-yaw angles to rotation matrix
    //
    // Calling Sequence
    // R = rpy2r(RPY, OPTIONS)
    // R = rpy2r(ROLL, PITCH, YAW, OPTIONS)
    //
    // Description
    // R = RPY2R(RPY, OPTIONS) is an orthonormal rotation matrix equivalent to the 
    // specified roll, pitch, yaw angles which correspond to rotations about the 
    // X, Y, Z axes respectively. If RPY has multiple rows they are assumed to 
    // represent a trajectory and R is a three dimensional matrix, where the last index  
    // corresponds to the rows of RPY.
    //
    // R = RPY2R(ROLL, PITCH, YAW, OPTIONS) as above but the roll-pitch-yaw angles 
    // are passed as separate arguments. If ROLL, PITCH and YAW are column vectors 
    // they are assumed to represent a trajectory and R is a three dimensional matrix,
    // where the last index corresponds to the rows of ROLL, PITCH, YAW.
    //
    // <refsection>
    //   <title>OPTIONS</title>
    //   <variablelist>
    //   <varlistentry><term>"deg" :</term>
    //         <listitem><para>Compute angles in degrees (radians default)</para></listitem></varlistentry>
    //  </variablelist>
    // </refsection>
    //
    //
    // See also 
    // tr2rpy
    // zyz2tr
    // eul2tr
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci
    // Copyright (C) 2014 - Corrado Guarino Lo Bianco
    
    [%nargout,%nargin] = argn(0);

    opt.deg = %f;
    if ~isempty(varargin) then
        for i = 1:(length(varargin)),
            if ~strcmp(string(varargin(i)),'deg')
                opt.deg = ~opt.deg;
            elseif typeof(varargin(i))=="string"
                warning(msprintf(gettext('%s: Unkwnon argument ''%s''\n'),'rpy2r',varargin(i)));
            end
        end
    end

    // unpack the arguments
    if numcols(roll) == 3
        pitch = roll(:,2);
        yaw = roll(:,3);
        roll = roll(:,1);
    elseif %nargin >= 3
        pitch = varargin(1);
        yaw = varargin(2);
    else
        error('bad arguments')
    end

    // optionally convert from degrees
    if opt.deg
        d2r = %pi/180.0;
        roll = roll * d2r;
        pitch = pitch * d2r;
        yaw = yaw * d2r;
    end

    if numrows(roll) == 1
        R = rotz(roll) * roty(pitch) * rotx(yaw);
    else
        R = zeros(3,3,numrows(roll));
        for i=1:numrows(roll)
            R(:,:,i) = rotz(roll(i)) * roty(pitch(i)) * rotx(yaw(i));
        end
    end
endfunction
