//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [R,t] = tr2rt(T)
    // Convert homogeneous transform to rotation and translation 
    //
    // Calling Sequence
    // [R,t] = tr2rt(TR)
    //    
    // Description  
    // [R,t] = TR2RT(TR) split a homogeneous transformation matrix (NxN) into an 
    // orthonormal rotation matrix R (MxM) and a translation vector t (Mx1), where
    // N=M+1.
    //
    // A homogeneous transform sequence TR (NxNxK) is split into rotation matrix 
    // sequence R (MxMxK) and a translation sequence t (KxM).
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para> Works for TR in SE(2) or SE(3) </para></listitem>
    // <listitem><para> If TR is 4x4, then R is 3x3 and T is 3x1. </para></listitem>
    // <listitem><para> If TR is 3x3, then R is 2x2 and T is 2x1.</para></listitem>
    // <listitem><para> The validity of R is not checked </para></listitem>
    // </itemizedlist>
    // </refsection>
    // 
    // See also 
    // rt2tr
    // r2t
    // t2r
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci
    
    if numcols(T) ~= numrows(T)
        error('T must be square');
    end

    n = numcols(T);

    if size(T,3) > 1
        R = zeros(3,3,size(T,3));
        t = zeros(size(T,3), 3);
        for i=1:size(T,3)
            R(:,:,i) = T(1:n-1,1:n-1,i);
            t(i,:) = T(1:n-1,n,i)';
        end
    else
        R = T(1:n-1,1:n-1);
        t = T(1:n-1,n);
    end
endfunction
