/*  Copyright (C) 2020 - Corrado Guarino Lo Bianco
*
*  This file is part of the Robotics Toolbox for Scilab.
*
*  Robotics Toolbox is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  Robotics Toolbox is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
*  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
*  by Peter Corke.
*/

#include "api_scilab.h"
#include "Scierror.h"
#include "malloc.h"
#include "string.h"
#include "t2rpy.h"
#include "r2rpy.h"
#include <localization.h>

/* ==================================================================== */
int rts_tr2rpy(char *fname, void* pvApiCtx)
{
    SciErr sciErr;

    double *newMatrixOfDouble = NULL;
    
    char string1[3] = "deg"; 
    char *string2;

    int i = 0;
    
    int m1 = 0, n1 = 0;
    int *piAddressVarOne = NULL;
    double *pdVarOne = NULL;
    int iType1 = 0;

    int m2 = 0, n2 = 0;
    int *piAddressVarTwo = NULL;
    double *pdVarTwo = NULL;
    int iType2 = 0;

    int m_out = 0, n_out = 0;
    double dOut = 0.0;

    /* check that we have only 3 input arguments */
    /* check that we have only 1 output argument */
    CheckInputArgument(pvApiCtx, 1, 2);
    CheckOutputArgument(pvApiCtx, 1, 1);

	switch (*getNbInputArgument(pvApiCtx))
    {
    case 1:
    	   /* get Address of inputs */
    	sciErr = getVarAddressFromPosition(pvApiCtx, 1, &piAddressVarOne);
    	if (sciErr.iErr)
    	{
    	      printError(&sciErr, 0);
    	      return 0;
    	}

    /* check input type */
    	sciErr = getVarType(pvApiCtx, piAddressVarOne, &iType1);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	if ( iType1 != sci_matrix )
    	{
        	Scierror(999, _("%s: Wrong type for input argument #%d: A matrix is expected.\n"), fname, 1);
        	return 0;
    	}

    
    /* get matrix */
    	sciErr = getMatrixOfDouble(pvApiCtx, piAddressVarOne, &m1, &n1, &pdVarOne);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    /* check size */
    	if ( m1 != 3 && m1 != 4)
    	{
        	Scierror(999, _("%s: Wrong size for input argument #%d: A three or a four rows matrix is expected.\n"), fname, 1);
        	return 0;
    	}
    	
    	if (((n1/4) != (n1/4.0)) && ((n1/3) != (n1/3.0)))
    	{
        	Scierror(999, _("%s: Wrong size for input argument #%d: A (4xn) or a (3xn) columns matrix is expected.\n"), fname, 1);
        	return 0;
    	}    	
    
		break;
		
	case 2:
	
    	   /* get Address of inputs */
    	sciErr = getVarAddressFromPosition(pvApiCtx, 1, &piAddressVarOne);
    	if (sciErr.iErr)
    	{
    	      printError(&sciErr, 0);
    	      return 0;
    	}

    	sciErr = getVarAddressFromPosition(pvApiCtx, 2, &piAddressVarTwo);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    /* check input type */
    	sciErr = getVarType(pvApiCtx, piAddressVarOne, &iType1);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	if ( iType1 != sci_matrix )
    	{
        	Scierror(999, _("%s: Wrong type for input argument #%d: A matrix expected.\n"), fname, 1);
        	return 0;
    	}

    	sciErr = getVarType(pvApiCtx, piAddressVarTwo, &iType2);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    	if ( iType2 != sci_strings )
    	{
        	Scierror(999, _("%s: Wrong type for input argument #%d: A string expected.\n"), fname, 2);
        	return 0;
    	}

    /* get matrix */
    	sciErr = getMatrixOfDouble(pvApiCtx, piAddressVarOne, &m1, &n1, &pdVarOne);
    	if (sciErr.iErr)
    	{
        	printError(&sciErr, 0);
        	return 0;
    	}

    /* check size */
    	if ( m1 != 3 && m1 != 4)
    	{
        	Scierror(999, _("%s: Wrong size for input argument #%d: A three or a four rows matrix is expected.\n"), fname, 1);
        	return 0;
    	}
    	
    	if (((n1/4) != (n1/4.0)) && ((n1/3) != (n1/3.0)))
    	{
        	Scierror(999, _("%s: Wrong size for input argument #%d: A (4xn) or a (3xn) columns matrix is expected.\n"), fname, 1);
        	return 0;
    	}

    /* get string */
    	getAllocatedSingleString(pvApiCtx, piAddressVarTwo, &string2);

    /* check string */
    	if ( strncmp(string1,string2,3) )
    	{
        	Scierror(999, _("%s: Wrong input argument #%d: ""deg"" is expected.\n"), fname, 2);
        	return 0;
    	}

		break;
		
   	default :
		Scierror(999, _("%s: Wrong number of input arguments: please consult the function help.\n"), fname);
        return 0;
	}
	
	
	if ((n1/4) == (n1/4.0))
	{
		m_out = n1/4;
	    newMatrixOfDouble = (int*)malloc(sizeof(double) * m_out * 3 );
	    /* call c function t2rpy */
	    for (i = 0; i < m_out; i++)
	    {
	        t2rpy(&pdVarOne[i*16], &newMatrixOfDouble[i], &newMatrixOfDouble[i+m_out], &newMatrixOfDouble[i+2*m_out]);
			/* convert in deg */
    		if ( *getNbInputArgument(pvApiCtx) == 2 )
    		{
    			newMatrixOfDouble[i] = newMatrixOfDouble[i]*180.0/M_PI;
    			newMatrixOfDouble[i+m_out] = newMatrixOfDouble[i+m_out]*180.0/M_PI;
    			newMatrixOfDouble[i+2*m_out] = newMatrixOfDouble[i+2*m_out]*180.0/M_PI;
    		}
    	}
	}
	else
	{
		m_out = n1/3;
	    newMatrixOfDouble = (int*)malloc(sizeof(double) * m_out * 3 );
	    /* call c function r2rpy */
	    for (i = 0; i < m_out; i++)
	    {
	        r2rpy(&pdVarOne[i*9], &newMatrixOfDouble[i], &newMatrixOfDouble[i+m_out], &newMatrixOfDouble[i+2*m_out]);
			/* convert in deg */
    		if ( *getNbInputArgument(pvApiCtx) == 2 )
    		{
    			newMatrixOfDouble[i] = newMatrixOfDouble[i]*180.0/M_PI;
    			newMatrixOfDouble[i+m_out] = newMatrixOfDouble[i+m_out]*180.0/M_PI;
    			newMatrixOfDouble[i+2*m_out] = newMatrixOfDouble[i+2*m_out]*180.0/M_PI;
    		}
    	}
	}

    /* create result on stack */
    n_out = 3;
    
    createMatrixOfDouble(pvApiCtx, nbInputArgument(pvApiCtx) + 1, m_out, n_out, newMatrixOfDouble);

    AssignOutputVariable(pvApiCtx, 1) = nbInputArgument(pvApiCtx) + 1;

    ReturnArguments(pvApiCtx);
    
    free(newMatrixOfDouble);
    
    return 0;
}
/* ==================================================================== */

