//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of the Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.


//TEST QUATERNION METHODS
// 
// Function checked:
// qinv
// qnorm
// qrot
// qhom
// qdouble
// qdot
// qscale
// qinterp
// and associated methods with the function "%Quat_e"


//Create a quaternion that represents a rotation of 90 degrees about X axis
quat = Quaternion(%pi/2, [1,0,0]);


//--------------------------------------------------------------------------//
//Check function: "qinv" and "quat.inv"
//--------------------------------------------------------------------------//
assert_checkalmostequal(quat.inv.s, sqrt(2)/2,1.e-15);
assert_checkalmostequal(quat.inv.v, [-sqrt(2)/2, 0, 0],1.e-15);
iquat = qinv(quat);
assert_checkalmostequal(iquat.s, sqrt(2)/2,1.e-15);
assert_checkalmostequal(iquat.v, [-sqrt(2)/2, 0, 0],1.e-15);


//--------------------------------------------------------------------------//
//Check function: "qnorm" and "quat.norm"
//--------------------------------------------------------------------------//
assert_checkequal(quat.norm ,1);
assert_checkequal(qnorm(quat) ,1);


//--------------------------------------------------------------------------//
//Check function: "qrot" and "quat.r"
//--------------------------------------------------------------------------//
T = [1, 0,  0 ; 0, 2.22D-16, -1; 0, 1,  2.22D-16];
T2 = qrot(quat);
for i=1,3 
    for j=1,3 
        assert_checkalmostequal(quat.r(i,j), T(i,j) ,1.e-15);
        assert_checkalmostequal(T2(i,j), T(i,j) ,1.e-15);
    end
end


//--------------------------------------------------------------------------//
//Check function: "qhom" and "quat.t"
//--------------------------------------------------------------------------//
T = [1, 0,  0, 0 ; 0, 2.22D-16, -1, 0 ; 0, 1,  2.22D-16, 0 ; 0, 0,  0, 1 ];
T2 = qhom(quat);
for i=1,4 
    for j=1,4 
        assert_checkalmostequal(quat.t(i,j), T(i,j) ,1.e-15);
        assert_checkalmostequal(T2(i,j), T(i,j) ,1.e-15);
    end
end


//--------------------------------------------------------------------------//
//Check function: "qdouble" and "quat.double"
//--------------------------------------------------------------------------//
assert_checkalmostequal(quat.double ,[sqrt(2)/2, sqrt(2)/2, 0, 0],1.e-15);
assert_checkalmostequal(qdouble(quat) ,[sqrt(2)/2, sqrt(2)/2, 0, 0],1.e-15);

//--------------------------------------------------------------------------//
//Check function: "qunit" and "quat.unit"
//--------------------------------------------------------------------------//
uquat = qunit(quat);
assert_checkalmostequal([quat.unit.s quat.unit.v] ,[sqrt(2)/2, sqrt(2)/2, 0, 0],1.e-15);
assert_checkalmostequal([uquat.s uquat.v] ,[sqrt(2)/2, sqrt(2)/2, 0, 0],1.e-15);


//--------------------------------------------------------------------------//
//Check function: "qdot"
//--------------------------------------------------------------------------//
dotquat = qdot(quat,[-1,1,%pi]);
assert_checkalmostequal(dotquat.s, 0.353553, 1.e-5);
assert_checkalmostequal(dotquat.v,[-0.353553, 1.464274, 0.757167], 1.e-5);


//--------------------------------------------------------------------------//
//Check function: "qscale"
//--------------------------------------------------------------------------//
r = 0.5;
scaledquat = qscale(quat,r);
assert_checkalmostequal(scaledquat.s, 0.923880, 1.e-5);
assert_checkalmostequal(scaledquat.v,[0.382683, 0, 0], 1.e-5);

//qscale returns a list
r = [0;0.5;1];
scaledquat = qscale(quat,r);
assert_checkequal(scaledquat(1).s, 1);
assert_checkequal(scaledquat(1).v,[0, 0, 0]);
assert_checkalmostequal(scaledquat(2).s, 0.923880, 1.e-5);
assert_checkalmostequal(scaledquat(2).v,[0.382683, 0, 0], 1.e-5);
assert_checkalmostequal(scaledquat(3).s, 0.707107, 1.e-5);
assert_checkalmostequal(scaledquat(3).v,[0.707107, 0, 0], 1.e-5);

scaledquat = qscale(quat,r');
assert_checkequal(scaledquat(1).s, 1);
assert_checkequal(scaledquat(1).v,[0, 0, 0]);
assert_checkalmostequal(scaledquat(2).s, 0.923880, 1.e-5);
assert_checkalmostequal(scaledquat(2).v,[0.382683, 0, 0], 1.e-5);
assert_checkalmostequal(scaledquat(3).s, 0.707107, 1.e-5);
assert_checkalmostequal(scaledquat(3).v,[0.707107, 0, 0], 1.e-5);

//--------------------------------------------------------------------------//
//Check function: "qinterp"
//--------------------------------------------------------------------------//
r = 0.5;
quat2 = Quaternion(%pi/2, [0,1,0]);
interpquat = qinterp(quat,quat2,r);
assert_checkalmostequal(interpquat.s, 0.816497, 1.e-5);
assert_checkalmostequal(interpquat.v,[0.408248, 0.408248, 0], 1.e-5);

//interpquat returns a list
r = [0;0.25;0.75;1];
interpquat = qinterp(quat,quat2,r);
assert_checkalmostequal(interpquat(1).s, 0.707107, 1.e-5);
assert_checkalmostequal(interpquat(1).v,[0.707107, 0, 0], 1.e-5);
assert_checkalmostequal(interpquat(2).s, 0.788675, 1.e-5);
assert_checkalmostequal(interpquat(2).v,[0.577350, 0.211325, 0], 1.e-5);
assert_checkalmostequal(interpquat(3).s, 0.788675, 1.e-5);
assert_checkalmostequal(interpquat(3).v,[0.211325, 0.577350, 0], 1.e-5);
assert_checkalmostequal(interpquat(4).s, 0.707107, 1.e-5);
assert_checkalmostequal(interpquat(4).v,[0, 0.707107, 0], 1.e-5);

assert_checktrue(qinterp(quat,quat2,r) == qinterp(quat,quat2,r'));

clear quat iquat uquat dotquat scaledquat interpquat quat2 T T2  i j r
